@@ -1,10 +1,10 @@
 /*
  * gpomme - GTK application for use with pommed
  *
- * $Id: gpomme.c 413 2007-12-04 11:15:51Z jblache $
+ * $Id: gpomme.c 502 2008-07-14 14:13:44Z jblache $
  *
- * Copyright (C) 2006 Soeren SONNENBURG <debian@nn7.de>
- * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
+ * Copyright (C) 2006, 2008 Soeren SONNENBURG <debian@nn7.de>
+ * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
  * Copyright (C) 2007 daniel g. siegel <dgsiegel@gmail.com>
  *
  * Portions of the GTK code below were shamelessly
@@ -48,6 +48,8 @@
 #include <X11/Xlib.h>
 
 #include <dbus/dbus.h>
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
 
 #include "gpomme.h"
 #include "theme.h"
@@ -70,12 +72,11 @@ struct
 DBusError dbus_err;
 DBusConnection *conn;
 
-/* inotify fd (config file monitoring) */
-int cfg_ifd = -1;
+dbus_uint32_t mute_serial = 0;
 
 
 /* Timer callback */
-gboolean
+static gboolean
 hide_window(gpointer userdata)
 {
   gtk_widget_hide(mbp_w.window);
@@ -85,8 +86,41 @@ hide_window(gpointer userdata)
   return FALSE;
 }
 
+static gboolean
+window_expose_event (GtkWidget *widget, GdkEventExpose *event)
+{
+  GdkRegion *region;
+  GtkWidget *child;
+  cairo_t *cr;
 
-void
+  /* get our child (in this case, the event box) */ 
+  child = gtk_bin_get_child (GTK_BIN (widget));
+
+  /* create a cairo context to draw to the window */
+  cr = gdk_cairo_create (widget->window);
+
+  /* the source data is the (composited) event box */
+  gdk_cairo_set_source_pixmap (cr, child->window,
+			       child->allocation.x,
+			       child->allocation.y);
+
+  /* draw no more than our expose event intersects our child */
+  region = gdk_region_rectangle (&child->allocation);
+  gdk_region_intersect (region, event->region);
+  gdk_cairo_region (cr, region);
+  cairo_clip (cr);
+
+  /* composite, with a 50% opacity */
+  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+  cairo_paint_with_alpha (cr, 1.0);
+
+  /* we're done */
+  cairo_destroy (cr);
+
+  return FALSE;
+}
+
+static void
 draw_window_bg(void)
 {
   GtkWidget *window = mbp_w.window;
@@ -115,7 +149,6 @@ draw_window_bg(void)
 
   gtk_window_move(GTK_WINDOW(window), x, y);
 
-
   /* Redraw the window background, compositing the background pixmap with
    * the portion of the root window that's beneath the window
    */
@@ -125,18 +158,23 @@ draw_window_bg(void)
 
   /* render the combined pixbuf to a pixmap with alpha control */
   pixmap = gdk_pixmap_new(GTK_WIDGET(window)->window, theme.width, theme.height, -1);
-  gdk_draw_pixbuf(pixmap, NULL, pixbuf, 0, 0, 0, 0,
-		  theme.width, theme.height, GDK_RGB_DITHER_NONE, 0, 0);
-  gdk_draw_pixbuf(pixmap, NULL, theme.background, 0, 0, 0, 0,
-		   theme.width, theme.height, GDK_RGB_DITHER_NONE, 0, 0);
+  GdkGC *gc = gdk_gc_new(pixmap);
+  gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, theme.width, theme.height);
+  if (!gdk_screen_is_composited(screen))
+  {
+    gdk_draw_pixbuf(pixmap, NULL, pixbuf, 0, 0, 0, 0,
+		    theme.width, theme.height, GDK_RGB_DITHER_NONE, 0, 0);
+  }
 
+  gdk_draw_pixbuf(pixmap, NULL, theme.background, 0, 0, 0, 0,
+		  theme.width, theme.height, GDK_RGB_DITHER_NONE, 0, 0);
   gdk_window_set_back_pixmap(GTK_WIDGET(window)->window, pixmap, FALSE);
 
   g_object_unref(pixbuf);
   g_object_unref(pixmap);
 }
 
-void
+static void
 show_window(int img, char *label, double fraction)
 {
   char *m_label;
@@ -204,18 +242,28 @@ show_window(int img, char *label, double fraction)
 }
 
 
-void
+static void
 create_window(void)
 {
+  GdkScreen *screen;
+  GdkColormap *rgba;
+
   GtkWidget *window;
   GtkWidget *vbox;
   GtkWidget *align;
 
   window = gtk_window_new(GTK_WINDOW_POPUP);
+  screen = gtk_widget_get_screen (window);
+
+  if (gdk_screen_is_composited (screen))
+    {
+      rgba = gdk_screen_get_rgba_colormap (screen);
+      gtk_widget_set_colormap (window, rgba);
+    }
+
   gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
   gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
   gtk_widget_set_app_paintable(GTK_WIDGET(window), TRUE);
-  gtk_widget_realize (GTK_WIDGET(window));
 
   gtk_window_set_default_size(GTK_WINDOW(window), theme.width, theme.height);
   gtk_widget_set_size_request(GTK_WIDGET(window), theme.width, theme.height);
@@ -230,7 +278,7 @@ create_window(void)
   /* Text message */
   align = gtk_alignment_new(0.5, 0.0, 0.0, 0.0);
   gtk_box_pack_start(GTK_BOX(vbox), align, TRUE, TRUE, 0);
-	
+
   mbp_w.label = gtk_label_new("");
   gtk_container_add(GTK_CONTAINER(align), mbp_w.label);
 
@@ -249,46 +297,132 @@ create_window(void)
   mbp_w.window = window;
   mbp_w.image = NULL;
   mbp_w.timer = 0;
+
+  gtk_widget_realize(GTK_WIDGET(window));
+
+  if (gdk_screen_is_composited(screen))
+    {
+      gdk_window_set_opacity(window->window, 0.8);
+      g_signal_connect_after(window, "expose-event", G_CALLBACK(window_expose_event), NULL);
+    }
 }
 
 
-void
-audio_getmute_cb(DBusPendingCall *pending, void *status)
+static gboolean
+mbp_dbus_reconnect(gpointer userdata);
+
+static DBusHandlerResult
+mbp_dbus_listen(DBusConnection *lconn, DBusMessage *msg, gpointer userdata)
 {
-  DBusMessage *msg;
+  int scratch;
+  int cur;
+  int max;
+  int who;
+  double ratio;
 
-  msg = dbus_pending_call_steal_reply(pending);
+  Display *dpy;
 
-  if (msg == NULL)
+  if (dbus_message_is_signal(msg, "org.pommed.signal.lcdBacklight", "lcdBacklight"))
     {
-      printf("Could not steal reply\n");
+      dbus_message_get_args(msg, &dbus_err,
+			    DBUS_TYPE_UINT32, &cur,
+			    DBUS_TYPE_UINT32, &scratch, /* previous */
+			    DBUS_TYPE_UINT32, &max,
+			    DBUS_TYPE_UINT32, &who,
+			    DBUS_TYPE_INVALID);
 
-      dbus_pending_call_unref(pending);
+      if (who == LCD_USER)
+	{
+	  ratio = (double)cur / (double)max;
 
-      return;
+	  show_window(IMG_LCD_BCK, _("LCD backlight level"), ratio);
+	}
     }
+  else if (dbus_message_is_signal(msg, "org.pommed.signal.kbdBacklight", "kbdBacklight"))
+    {
+      dbus_message_get_args(msg, &dbus_err,
+			    DBUS_TYPE_UINT32, &cur,
+			    DBUS_TYPE_UINT32, &scratch, /* previous */
+			    DBUS_TYPE_UINT32, &max,
+			    DBUS_TYPE_UINT32, &who,
+			    DBUS_TYPE_INVALID);
 
-  dbus_pending_call_unref(pending);
+      if (who == KBD_USER)
+	{
+	  ratio = (double)cur / (double)max;
 
-  if (!mbp_dbus_check_error(msg))
+	  show_window(IMG_KBD_BCK, _("Keyboard backlight level"), ratio);
+	}
+    }
+  else if (dbus_message_is_signal(msg, "org.pommed.signal.audioVolume", "audioVolume"))
+    {
+      dbus_message_get_args(msg, &dbus_err,
+			    DBUS_TYPE_UINT32, &cur,
+			    DBUS_TYPE_UINT32, &scratch, /* previous */
+			    DBUS_TYPE_UINT32, &max,
+			    DBUS_TYPE_INVALID);
+
+      ratio = (double)cur / (double)max;
+
+      if (!mbp.muted)
+	show_window(IMG_AUDIO_VOL_ON, _("Sound volume"), ratio);
+      else
+	show_window(IMG_AUDIO_VOL_OFF, _("Sound volume (muted)"), ratio);
+    }
+  else if (dbus_message_is_signal(msg, "org.pommed.signal.audioMute", "audioMute"))
     {
       dbus_message_get_args(msg, &dbus_err,
 			    DBUS_TYPE_BOOLEAN, &mbp.muted,
 			    DBUS_TYPE_INVALID);
+
+      if (mbp.muted)
+	show_window(IMG_AUDIO_MUTE, _("Sound muted"), -1.0);
+      else
+	show_window(IMG_AUDIO_MUTE, _("Sound unmuted"), -1.0);
+    }
+  else if (dbus_message_is_signal(msg, "org.pommed.signal.cdEject", "cdEject"))
+    {
+      show_window(IMG_CD_EJECT, _("Eject"), -1.0);
+    }
+  else if (dbus_message_is_signal(msg, "org.pommed.signal.videoSwitch", "videoSwitch"))
+    {
+      dpy = GDK_WINDOW_XDISPLAY(GTK_WIDGET(mbp_w.window)->window);
+      mbp_video_switch(dpy);
+    }
+  else if (dbus_message_is_signal(msg, DBUS_INTERFACE_LOCAL, "Disconnected"))
+    {
+      printf("DBus disconnected\n");
+
+      mbp_dbus_cleanup();
+
+      g_timeout_add(200, mbp_dbus_reconnect, NULL);
     }
   else
-    *(int *)status = -1;
+    {
+      if ((dbus_message_get_type(msg) == DBUS_MESSAGE_TYPE_METHOD_RETURN)
+	  && (dbus_message_get_reply_serial(msg) == mute_serial))
+	{
+	  dbus_message_get_args(msg, &dbus_err,
+				DBUS_TYPE_BOOLEAN, &mbp.muted,
+				DBUS_TYPE_INVALID);
+	}
+      else
+	{
+	  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+    }
 
-  dbus_message_unref(msg);
+  return DBUS_HANDLER_RESULT_HANDLED;
 }
 
-
-int
+static int
 mbp_dbus_connect(void)
 {
   unsigned int signals;
+
+  DBusMessage *msg;
+
   int ret;
-  int cbret;
 
   signals = MBP_DBUS_SIG_LCD | MBP_DBUS_SIG_KBD
     | MBP_DBUS_SIG_VOL | MBP_DBUS_SIG_MUTE
@@ -299,143 +433,57 @@ mbp_dbus_connect(void)
   if (conn == NULL)
     return -1;
 
-  ret = mbp_call_audio_getmute(audio_getmute_cb, &cbret);
-  if ((ret < 0) || (cbret < 0))
+  dbus_connection_setup_with_g_main(conn, NULL);
+
+  dbus_connection_add_filter(conn, mbp_dbus_listen, NULL, NULL);
+
+  /* Get the mute state */
+  msg = dbus_message_new_method_call("org.pommed", "/org/pommed/audio",
+				     "org.pommed.audio", "getMute");
+
+  if (msg == NULL)
     {
-      printf("audio getMute call failed !\n");
+      printf("Failed to create method call message for audio getMute\n");
+
+      return 0;
     }
 
+  ret = dbus_connection_send(conn, msg, &mute_serial);
+  if (ret == FALSE)
+    {
+      printf("Could not send method call for audio getMute\n");
+
+      dbus_message_unref(msg);
+
+      return 0;
+    }
+
+  dbus_connection_flush(conn);
+
+  dbus_message_unref(msg);
+
   return 0;
 }
 
-
-void
-mbp_dbus_listen(gpointer userdata)
+static gboolean
+mbp_dbus_reconnect(gpointer userdata)
 {
-  DBusMessage *msg;
+  if (mbp_dbus_connect() < 0)
+    return TRUE;
 
-  int ret;
-
-  int scratch;
-  int cur;
-  int max;
-  int who;
-  double ratio;
-
-  Display *dpy;
-
-  /* Disconnected, try to reconnect */
-  if (conn == NULL)
-    {
-      ret = mbp_dbus_connect();
-
-      if (ret < 0)
-	return;
-    }
-
-  while (1)
-    {
-      dbus_connection_read_write(conn, 0);
-
-      msg = dbus_connection_pop_message(conn);
-
-      if (msg == NULL)
-	return;
-
-      if (dbus_message_is_signal(msg, "org.pommed.signal.lcdBacklight", "lcdBacklight"))
-	{
-	  dbus_message_get_args(msg, &dbus_err,
-				DBUS_TYPE_UINT32, &cur,
-				DBUS_TYPE_UINT32, &scratch, /* previous */
-				DBUS_TYPE_UINT32, &max,
-				DBUS_TYPE_UINT32, &who,
-				DBUS_TYPE_INVALID);
-
-	  if (who == LCD_USER)
-	    {
-	      ratio = (double)cur / (double)max;
-
-	      show_window(IMG_LCD_BCK, _("LCD backlight level"), ratio);
-	    }
-	}
-      else if (dbus_message_is_signal(msg, "org.pommed.signal.kbdBacklight", "kbdBacklight"))
-	{
-	  dbus_message_get_args(msg, &dbus_err,
-				DBUS_TYPE_UINT32, &cur,
-				DBUS_TYPE_UINT32, &scratch, /* previous */
-				DBUS_TYPE_UINT32, &max,
-				DBUS_TYPE_UINT32, &who,
-				DBUS_TYPE_INVALID);
-
-	  if (who == KBD_USER)
-	    {
-	      ratio = (double)cur / (double)max;
-
-	      show_window(IMG_KBD_BCK, _("Keyboard backlight level"), ratio);
-	    }
-	}
-      else if (dbus_message_is_signal(msg, "org.pommed.signal.audioVolume", "audioVolume"))
-	{
-	  dbus_message_get_args(msg, &dbus_err,
-				DBUS_TYPE_UINT32, &cur,
-				DBUS_TYPE_UINT32, &scratch, /* previous */
-				DBUS_TYPE_UINT32, &max,
-				DBUS_TYPE_INVALID);
-
-	  ratio = (double)cur / (double)max;
-
-	  if (!mbp.muted)
-	    show_window(IMG_AUDIO_VOL_ON, _("Sound volume"), ratio);
-	  else
-	    show_window(IMG_AUDIO_VOL_OFF, _("Sound volume (muted)"), ratio);
-	}
-      else if (dbus_message_is_signal(msg, "org.pommed.signal.audioMute", "audioMute"))
-	{
-	  dbus_message_get_args(msg, &dbus_err,
-				DBUS_TYPE_BOOLEAN, &mbp.muted,
-				DBUS_TYPE_INVALID);
-
-	  if (mbp.muted)
-	    show_window(IMG_AUDIO_MUTE, _("Sound muted"), -1.0);
-	  else
-	    show_window(IMG_AUDIO_MUTE, _("Sound unmuted"), -1.0);
-	}
-      else if (dbus_message_is_signal(msg, "org.pommed.signal.cdEject", "cdEject"))
-	{
-	  show_window(IMG_CD_EJECT, _("Eject"), -1.0);
-	}
-      else if (dbus_message_is_signal(msg, "org.pommed.signal.videoSwitch", "videoSwitch"))
-	{
-	  dpy = GDK_WINDOW_XDISPLAY(GTK_WIDGET(mbp_w.window)->window);
-	  mbp_video_switch(dpy);
-	}
-      else if (dbus_message_is_signal(msg, DBUS_INTERFACE_LOCAL, "Disconnected"))
-	{
-	  printf("DBus disconnected\n");
-
-	  mbp_dbus_cleanup();
-
-	  dbus_message_unref(msg);
-
-	  break;
-	}
-
-      dbus_message_unref(msg);
-    }
+  return FALSE;
 }
 
-void
-mbp_check_config(gpointer userdata)
-{
-  struct inotify_event ie;
 
+static gboolean
+mbp_check_config(GIOChannel *ch, GIOCondition condition, gpointer userdata)
+{
+  int fd;
   int ret;
 
-  if (cfg_ifd < 0)
-    return;
+  fd = g_io_channel_unix_get_fd(ch);
 
-  ret = read(cfg_ifd, &ie, sizeof(struct inotify_event));
-  if (ret > 0)
+  if (condition & G_IO_IN)
     {
       ret = config_load();
       if (ret < 0)
@@ -444,28 +492,28 @@ mbp_check_config(gpointer userdata)
 
 	  gtk_main_quit();
 	}
-
-      close(cfg_ifd);
-
-      cfg_ifd = config_monitor();
     }
-}
 
-gboolean
-mbp_check_events(gpointer userdata)
-{
-  mbp_check_config(userdata);
-  mbp_dbus_listen(userdata);
+  close(fd);
 
-  return TRUE;
+  fd = config_monitor();
+
+  if (fd > 0)
+    {
+      ch = g_io_channel_unix_new(fd);
+      g_io_channel_set_encoding(ch, NULL, NULL);
+      g_io_add_watch(ch, G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL, mbp_check_config, NULL);
+    }
+
+  return FALSE;
 }
 
 
 static void
 usage(void)
 {
-  printf("gpomme v" M_VERSION " ($Rev: 413 $) graphical client for pommed\n");
-  printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org> and others\n");
+  printf("gpomme v" M_VERSION " ($Rev: 502 $) graphical client for pommed\n");
+  printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org> and others\n");
 
   printf("Usage:\n");
   printf("\tgpomme\t\t-- start gpomme\n");
@@ -474,7 +522,8 @@ usage(void)
 }
 
 
-void sig_int_term_handler(int signo)
+void
+sig_int_term_handler(int signo)
 {
   gtk_main_quit();
 }
@@ -491,11 +540,15 @@ sig_chld_handler(int signo)
   while (ret > 0);
 }
 
-int main(int argc, char **argv)
+int
+main(int argc, char **argv)
 {
   int c;
   int ret;
 
+  GIOChannel *ch;
+  int fd;
+
   gtk_init(&argc, &argv);
 
   bindtextdomain("gpomme", "/usr/share/locale");
@@ -520,8 +573,8 @@ int main(int argc, char **argv)
 	    break;
 
 	  case 'v':
-	    printf("gpomme v" M_VERSION " ($Rev: 413 $) graphical client for pommed\n");
-	    printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org> and others\n");
+	    printf("gpomme v" M_VERSION " ($Rev: 502 $) graphical client for pommed\n");
+	    printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org> and others\n");
 
 	    exit(0);
 	    break;
@@ -536,7 +589,14 @@ int main(int argc, char **argv)
 
   mbp_dbus_connect();
 
-  cfg_ifd = config_monitor();
+  fd = config_monitor();
+
+  if (fd > 0)
+    {
+      ch = g_io_channel_unix_new(fd);
+      g_io_channel_set_encoding(ch, NULL, NULL);
+      g_io_add_watch(ch, G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL, mbp_check_config, NULL);
+    }
 
   signal(SIGINT, sig_int_term_handler);
   signal(SIGTERM, sig_int_term_handler);
@@ -544,12 +604,8 @@ int main(int argc, char **argv)
 
   create_window();
 
-  g_timeout_add(100, mbp_check_events, NULL);
-
   gtk_main();
 
-  close(cfg_ifd);
-
   mbp_dbus_cleanup();
 
   return 0;
