@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: audio.c 317 2007-04-28 20:13:45Z jblache $
+ * $Id: audio.c 387 2007-11-25 13:27:01Z jblache $
  *
  * Copyright (C) 2006 Romain Beauxis <toots@rastageeks.org>
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
@@ -23,11 +23,13 @@
 #include <stdio.h>
 #include <string.h>
 
+#define NDEBUG
 #include <alsa/asoundlib.h>
 
 #include "pommed.h"
 #include "conffile.h"
 #include "audio.h"
+#include "beep.h"
 #include "dbus.h"
 
 
@@ -91,6 +93,9 @@ audio_step(int dir)
   if (snd_mixer_selem_is_playback_mono(vol_elem) == 0)
     snd_mixer_selem_set_playback_volume(vol_elem, 1, newvol);
 
+  if (audio_cfg.beep)
+    beep_audio();
+
   mbpdbus_send_audio_volume(newvol, vol);
 
   audio_info.level = newvol;
