@@ -1,33 +1,17 @@
 /*
- * $Id: beep.h 427 2007-12-12 11:39:26Z jblache $
+ * $Id: beep.h 481 2008-06-02 18:21:57Z jblache $
  */
 
 #ifndef __BEEP_H__
 #define __BEEP_H__
 
-struct _beep_info
-{
-  int fd;
-};
-
-extern struct _beep_info beep_info;
-
 
 #define BEEP_DEFAULT_FILE    "/usr/share/pommed/goutte.wav"
 #define BEEP_DEVICE_NAME     "Pommed beeper device"
 
-void
-beep_beep(void);
-
 void
 beep_audio(void);
 
-int
-beep_open_device(void);
-
-void
-beep_close_device(void);
-
 int
 beep_init(void);
 
