@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: cd_eject.c 329 2007-05-12 15:39:27Z jblache $
+ * $Id: cd_eject.c 508 2008-07-22 09:55:58Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -97,7 +97,7 @@ cd_eject(void)
     }
   else if (ret == -1)
     {
-      logmsg(LOG_ERR, "Could not fork: %s\n", strerror(errno));
+      logmsg(LOG_ERR, "Could not fork: %s", strerror(errno));
       return;
     }
   else
