@@ -1,5 +1,5 @@
 /*
- * $Id: conffile.h 387 2007-11-25 13:27:01Z jblache $
+ * $Id: conffile.h 525 2008-12-11 14:35:53Z jblache $
  */
 
 #ifndef __CONFFILE_H__
@@ -9,15 +9,14 @@ struct _general_cfg {
   int fnmode;
 };
 
-#ifdef __powerpc__
 struct _lcd_sysfs_cfg {
   int init;
   int step;
   int on_batt;
 };
 
-#else
 
+#ifndef __powerpc__
 struct _lcd_x1600_cfg {
   int init;
   int step;
@@ -35,8 +34,7 @@ struct _lcd_nv8600mgt_cfg {
   int step;
   int on_batt;
 };
-
-#endif /* __powerpc__ */
+#endif /* !__powerpc__ */
 
 struct _audio_cfg {
   char *card;
@@ -75,9 +73,8 @@ struct _appleir_cfg {
 
 
 extern struct _general_cfg general_cfg;
-#ifdef __powerpc__
 extern struct _lcd_sysfs_cfg lcd_sysfs_cfg;
-#else
+#ifndef __powerpc__
 extern struct _lcd_x1600_cfg lcd_x1600_cfg;
 extern struct _lcd_gma950_cfg lcd_gma950_cfg;
 extern struct _lcd_nv8600mgt_cfg lcd_nv8600mgt_cfg;
