@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: dbus.c 488 2008-06-07 09:48:28Z jblache $
+ * $Id: dbus.c 527 2008-12-11 14:56:35Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -1075,7 +1075,7 @@ mbpdbus_init(void)
 
       conn = NULL;
 
-      return -1;
+      goto init_reconnect;
     }
 
   dbus_connection_set_exit_on_disconnect(conn, FALSE);
@@ -1088,7 +1088,7 @@ mbpdbus_init(void)
 
       mbpdbus_cleanup();
 
-      return -1;
+      goto init_reconnect;
     }
 
   if (ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
@@ -1097,7 +1097,7 @@ mbpdbus_init(void)
 
       mbpdbus_cleanup();
 
-      return -1;
+      goto init_reconnect;
     }
 
   ret = dbus_connection_set_watch_functions(conn, mbpdbus_add_watch, mbpdbus_remove_watch,
@@ -1106,12 +1106,26 @@ mbpdbus_init(void)
     {
       mbpdbus_cleanup();
 
-      return -1;
+      goto init_reconnect;
     }
 
   dbus_connection_add_filter(conn, mbpdbus_process_requests, NULL, NULL);
 
   return 0;
+
+ init_reconnect:
+  if (dbus_timer > 0) /* Trying to reconnect already, called from mbpdbus_reconnect() */
+      return -1;
+
+  dbus_timer = evloop_add_timer(DBUS_TIMEOUT, mbpdbus_reconnect);
+  if (dbus_timer < 0)
+    {
+      logmsg(LOG_ERR, "Could not set up timer for DBus reconnection");
+
+      return -1;
+    }
+
+  return 0;
 }
 
 void
