@@ -1,10 +1,12 @@
 /*
- * $Id: dbus.h 388 2007-11-26 10:31:11Z jblache $
+ * $Id: dbus.h 482 2008-06-03 17:19:03Z jblache $
  */
 
 #ifndef __MBPDBUS_H__
 #define __MBPDBUS_H__
 
+#define DBUS_TIMEOUT 200
+
 
 void
 mbpdbus_send_lcd_backlight(int cur, int prev, int who);
@@ -28,10 +30,6 @@ void
 mbpdbus_send_video_switch(void);
 
 
-void
-mbpdbus_process_requests(void);
-
-
 int
 mbpdbus_init(void);
 
