@@ -1,9 +1,9 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: evdev.c 500 2008-07-13 17:53:17Z jblache $
+ * $Id: evdev.c 541 2009-01-03 18:07:38Z jblache $
  *
- * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
+ * Copyright (C) 2006-2009 Julien BLACHE <jb@jblache.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -571,6 +571,34 @@ evdev_is_wellspring2(unsigned short *id)
   return 0;
 }
 
+/* Core2 Duo MacBook Pro (MacBookPro5,1, October 2008)
+ * Core2 Duo MacBook (MacBook5,1, October 2008)
+ * MacBook Air (MacBookAir2,1, October 2008) */
+static int
+evdev_is_wellspring3(unsigned short *id)
+{
+  unsigned short product = id[ID_PRODUCT];
+
+  if (id[ID_BUS] != BUS_USB)
+    return 0;
+
+  if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
+    return 0;
+
+  if ((product == USB_PRODUCT_ID_WELLSPRING3_ANSI)
+      || (product == USB_PRODUCT_ID_WELLSPRING3_ISO)
+      || (product == USB_PRODUCT_ID_WELLSPRING3_JIS))
+    {
+      logdebug(" -> WellSpring III USB assembly\n");
+
+      kbd_set_fnmode();
+
+      return 1;
+    }
+
+  return 0;
+}
+
 /* Any internal keyboard */
 static int
 evdev_is_internal(unsigned short *id)
@@ -579,7 +607,8 @@ evdev_is_internal(unsigned short *id)
 	  || evdev_is_geyser4(id)
 	  || evdev_is_geyser4hf(id)
 	  || evdev_is_wellspring(id)
-	  || evdev_is_wellspring2(id));
+	  || evdev_is_wellspring2(id)
+	  || evdev_is_wellspring3(id));
 }
 
 
@@ -785,7 +814,8 @@ evdev_try_add(int fd)
 	  return -1;
 	}
     }
-  else if (test_bit(EV_ABS, bit[0]))
+  /* Wireless keyboards advertise EV_ABS events, single them out */
+  else if (test_bit(EV_ABS, bit[0]) && !(evdev_is_extkbd_alu_wl(id)))
     {
       logdebug("Discarding evdev with EV_ABS event type (mouse/trackpad)\n");
 
