@@ -3,7 +3,7 @@
  *
  * Apple Macbook Pro LCD backlight control
  *
- * $Id: x1600_backlight.c 344 2007-07-27 13:52:53Z jblache $
+ * $Id: x1600_backlight.c 538 2008-12-29 12:22:15Z jblache $
  *
  * Copyright (C) 2006 Nicolas Boichat <nicolas@boichat.ch>
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
@@ -177,6 +177,7 @@ x1600_backlight_step(int dir)
 void
 x1600_backlight_toggle(int lvl)
 {
+  int val;
   int ret;
 
   if (lcd_x1600_cfg.on_batt == 0)
@@ -186,9 +187,25 @@ x1600_backlight_toggle(int lvl)
   if (ret < 0)
     return;
 
+  val = x1600_backlight_get();
+  if (val != lcd_bck_info.level)
+    {
+      mbpdbus_send_lcd_backlight(val, lcd_bck_info.level, LCD_AUTO);
+      lcd_bck_info.level = val;
+    }
+
+  if (lcd_bck_info.level == 0)
+    {
+      x1600_backlight_unmap();
+      return;
+    }
+
   switch (lvl)
     {
       case LCD_ON_AC_LEVEL:
+	if (lcd_bck_info.level >= lcd_bck_info.ac_lvl)
+	  break;
+
 	logdebug("LCD switching to AC level\n");
 
 	x1600_backlight_set(lcd_bck_info.ac_lvl);
@@ -199,17 +216,18 @@ x1600_backlight_toggle(int lvl)
 	break;
 
       case LCD_ON_BATT_LEVEL:
+	if (lcd_bck_info.level <= lcd_x1600_cfg.on_batt)
+	  break;
+
 	logdebug("LCD switching to battery level\n");
 
 	lcd_bck_info.ac_lvl = lcd_bck_info.level;
 
-	if (lcd_bck_info.level > lcd_x1600_cfg.on_batt)
-	  {
-	    x1600_backlight_set(lcd_x1600_cfg.on_batt);
-	    lcd_bck_info.level = lcd_x1600_cfg.on_batt;
+	x1600_backlight_set(lcd_x1600_cfg.on_batt);
 
-	    mbpdbus_send_lcd_backlight(lcd_bck_info.level, lcd_bck_info.ac_lvl, LCD_AUTO);
-	  }
+	mbpdbus_send_lcd_backlight(lcd_x1600_cfg.on_batt, lcd_bck_info.level, LCD_AUTO);
+
+	lcd_bck_info.level = lcd_x1600_cfg.on_batt;
 	break;
     }
 
