@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: kbd_backlight.c 482 2008-06-03 17:19:03Z jblache $
+ * $Id: kbd_backlight.c 508 2008-07-22 09:55:58Z jblache $
  *
  * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
  * Copyright (C) 2006 Yves-Alexis Perez <corsac@corsac.net>
@@ -64,6 +64,29 @@ kbd_backlight_get(void)
 }
 
 
+/* Helper for LMU-controlled keyboards */
+static void
+lmu_write_kbd_value(int fd, unsigned char val)
+{
+  unsigned char buf[3];
+
+  buf[0] = 0x01;   /* i2c register */
+
+  /* The format appears to be: (taken from pbbuttonsd)
+   *          byte 1   byte 2
+   *         |<---->| |<---->|
+   *         xxxx7654 3210xxxx
+   *             |<----->|
+   *                 ^-- brightness
+   */
+
+  buf[1] = val >> 4;
+  buf[2] = val << 4;
+
+  if (write (fd, buf, 3) < 0)
+    logmsg(LOG_ERR, "Could not set LMU kbd brightness: %s", strerror(errno));
+}
+
 static void
 kbd_lmu_backlight_set(int val, int who)
 {
@@ -76,7 +99,6 @@ kbd_lmu_backlight_set(int val, int who)
 
   int fd;
   int ret;
-  unsigned char buf[8];
 
   if (kbd_bck_info.inhibit & ~KBD_INHIBIT_CFG)
     return;
@@ -95,7 +117,7 @@ kbd_lmu_backlight_set(int val, int who)
   fd = open(lmu_info.i2cdev, O_RDWR);
   if (fd < 0)
     {
-      logmsg(LOG_ERR, "Could not open %s: %s\n", lmu_info.i2cdev, strerror(errno));
+      logmsg(LOG_ERR, "Could not open %s: %s", lmu_info.i2cdev, strerror(errno));
 
       return;
     }
@@ -103,14 +125,12 @@ kbd_lmu_backlight_set(int val, int who)
   ret = ioctl(fd, I2C_SLAVE, lmu_info.lmuaddr);
   if (ret < 0)
     {
-      logmsg(LOG_ERR, "Could not ioctl the i2c bus: %s\n", strerror(errno));
+      logmsg(LOG_ERR, "Could not ioctl the i2c bus: %s", strerror(errno));
 
       close(fd);
       return;
     }
 
-  buf[0] = 0x01;   /* i2c register */
-
   if (who == KBD_AUTO)
     {
       fade_step.tv_sec = 0;
@@ -123,36 +143,15 @@ kbd_lmu_backlight_set(int val, int who)
 	{
 	  fadeval += step;
 
-	  /* See below for the format */
-	  buf[1] = (unsigned char) fadeval >> 4;
-	  buf[2] = (unsigned char) fadeval << 4;
-
-	  if (write (fd, buf, 3) < 0)
-	    {
-	      logmsg(LOG_ERR, "Could not set LMU kbd brightness: %s\n", strerror(errno));
-
-	      continue;
-	    }
+	  lmu_write_kbd_value(fd, (unsigned char)fadeval);
 
 	  logdebug("KBD backlight value faded to %d\n", (int)fadeval);
 
 	  nanosleep(&fade_step, NULL);
 	}
     }
-  
-  /* The format appears to be: (taken from pbbuttonsd)
-   *          byte 1   byte 2
-   *         |<---->| |<---->|
-   *         xxxx7654 3210xxxx
-   *             |<----->|
-   *                 ^-- brightness
-   */
-  
-  buf[1] = (unsigned char) val >> 4;
-  buf[2] = (unsigned char) val << 4;
 
-  if (write (fd, buf, 3) < 0)
-    logmsg(LOG_ERR, "Could not set LMU kbd brightness: %s\n", strerror(errno));
+  lmu_write_kbd_value(fd, val);
 
   close(fd);
 
@@ -161,6 +160,33 @@ kbd_lmu_backlight_set(int val, int who)
   kbd_bck_info.level = val;
 }
 
+
+/* Helper for ADB keyboards */
+static void
+adb_write_kbd_value(int fd, unsigned char val)
+{
+  int ret;
+  unsigned char buf[ADB_BUFFER_SIZE];
+
+  buf[0] = PMU_PACKET;
+  buf[1] = 0x4f; /* PMU command */
+  buf[2] = 0;
+  buf[3] = 0;
+  buf[4] = val;
+
+  ret = write(fd, buf, 5);
+  if (ret != 5)
+    {
+      logmsg(LOG_ERR, "Could not set PMU kbd brightness: %s", strerror(errno));
+    }
+  else
+    {
+      ret = read(fd, buf, ADB_BUFFER_SIZE);
+      if (ret < 0)
+	logmsg(LOG_ERR, "Could not read PMU reply: %s", strerror(errno));
+    }
+}
+
 static void
 kbd_pmu_backlight_set(int val, int who)
 {
@@ -172,8 +198,6 @@ kbd_pmu_backlight_set(int val, int who)
   struct timespec fade_step;
 
   int fd;
-  int ret;
-  unsigned char buf[ADB_BUFFER_SIZE];
 
   if (kbd_bck_info.inhibit & ~KBD_INHIBIT_CFG)
     return;
@@ -189,7 +213,7 @@ kbd_pmu_backlight_set(int val, int who)
   fd = open(ADB_DEVICE, O_RDWR);
   if (fd < 0)
     {
-      logmsg(LOG_ERR, "Could not open %s: %s\n", ADB_DEVICE, strerror(errno));
+      logmsg(LOG_ERR, "Could not open %s: %s", ADB_DEVICE, strerror(errno));
 
       return;
     }
@@ -206,51 +230,15 @@ kbd_pmu_backlight_set(int val, int who)
 	{
 	  fadeval += step;
 
-	  buf[0] = PMU_PACKET;
-	  buf[1] = 0x4f; /* PMU command */
-	  buf[2] = 0;
-	  buf[3] = 0;
-	  buf[4] = (unsigned char)fadeval;
-
-	  ret = write(fd, buf, 5);
-	  if (ret != 5)
-	    {
-	      logmsg(LOG_ERR, "Could not set PMU kbd brightness: %s\n", strerror(errno));
-
-	      continue;
-	    }
-
-	  ret = read(fd, buf, ADB_BUFFER_SIZE);
-	  if (ret < 0)
-	    {
-	      logmsg(LOG_ERR, "Could not read PMU reply: %s\n", strerror(errno));
-
-	      continue;
-	    }
+	  adb_write_kbd_value(fd, (unsigned char)val);
 
 	  logdebug("KBD backlight value faded to %d\n", (int)fadeval);
 
 	  nanosleep(&fade_step, NULL);
 	}
     }
-  
-  buf[0] = PMU_PACKET;
-  buf[1] = 0x4f; /* PMU command */
-  buf[2] = 0;
-  buf[3] = 0;
-  buf[4] = val;
 
-  ret = write(fd, buf, 5);
-  if (ret != 5)
-    {
-      logmsg(LOG_ERR, "Could not set PMU kbd brightness: %s\n", strerror(errno));
-    }
-  else
-    {
-      ret = read(fd, buf, ADB_BUFFER_SIZE);
-      if (ret < 0)
-	logmsg(LOG_ERR, "Could not read PMU reply: %s\n", strerror(errno));
-    }
+  adb_write_kbd_value(fd, val);
 
   close(fd);
 
@@ -336,7 +324,8 @@ kbd_backlight_init(void)
 
   kbd_bck_info.auto_on = 0;
 
-  if ((mops->type == MACHINE_POWERBOOK_58)
+  if (!has_kbd_backlight()
+      || (mops->type == MACHINE_POWERBOOK_58)
       || (mops->type == MACHINE_POWERBOOK_59))
     {
       /* Nothing to probe for the PMU05 machines */
@@ -345,7 +334,7 @@ kbd_backlight_init(void)
   else
     ret = kbd_probe_lmu();
 
-  if ((!has_kbd_backlight()) || (ret < 0))
+  if (!has_kbd_backlight() || (ret < 0))
     {
       lmu_info.lmuaddr = 0;
 
@@ -510,7 +499,7 @@ kbd_probe_lmu(void)
   fd = open(lmu_info.i2cdev, O_RDWR);
   if (fd < 0)
     {
-      logmsg(LOG_WARNING, "Could not open device %s: %s\n", lmu_info.i2cdev, strerror(errno));
+      logmsg(LOG_WARNING, "Could not open device %s: %s", lmu_info.i2cdev, strerror(errno));
 
       return -1;
     }
@@ -518,7 +507,7 @@ kbd_probe_lmu(void)
   ret = ioctl(fd, I2C_SLAVE, lmu_info.lmuaddr);
   if (ret < 0)
     {
-      logmsg(LOG_ERR, "ioctl failed on %s: %s\n", lmu_info.i2cdev, strerror(errno));
+      logmsg(LOG_ERR, "ioctl failed on %s: %s", lmu_info.i2cdev, strerror(errno));
 
       close(fd);
       return -1;
@@ -527,7 +516,7 @@ kbd_probe_lmu(void)
   ret = read(fd, buffer, 4);
   if (ret != 4)
     {
-      logmsg(LOG_WARNING, "Probing failed on %s: %s\n", lmu_info.i2cdev, strerror(errno));
+      logmsg(LOG_WARNING, "Probing failed on %s: %s", lmu_info.i2cdev, strerror(errno));
 
       close(fd);
       return -1;
