@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: pommed.c 494 2008-06-12 19:18:53Z jblache $
+ * $Id: pommed.c 540 2009-01-02 20:21:27Z jblache $
  *
  * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
  *
@@ -30,6 +30,8 @@
 #include <string.h>
 #include <signal.h>
 
+#include <sys/utsname.h>
+
 #include <syslog.h>
 #include <stdarg.h>
 
@@ -261,20 +263,28 @@ struct machine_ops mb_mops[] = {
 
   {  /* MacBookPro3,1 (15" & 17", Core2 Duo, June 2007) */
     .type = MACHINE_MACBOOKPRO_3,
-    .lcd_backlight_probe = nv8600mgt_backlight_probe,
-    .lcd_backlight_step = nv8600mgt_backlight_step,
-    .lcd_backlight_toggle = nv8600mgt_backlight_toggle,
+    .lcd_backlight_probe = mbp_sysfs_backlight_probe,
+    .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     /* .evdev_identify = evdev_is_geyser4, */
   },
 
   {  /* MacBookPro4,1 (15" & 17", Core2 Duo, February 2008) */
     .type = MACHINE_MACBOOKPRO_4,
-    .lcd_backlight_probe = nv8600mgt_backlight_probe,
-    .lcd_backlight_step = nv8600mgt_backlight_step,
-    .lcd_backlight_toggle = nv8600mgt_backlight_toggle,
+    .lcd_backlight_probe = mbp_sysfs_backlight_probe,
+    .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     /* .evdev_identify = evdev_is_wellspring2, */
   },
 
+  {  /* MacBookPro5,1 (15" & 17", Core2 Duo, October 2008) */
+    .type = MACHINE_MACBOOKPRO_5,
+    .lcd_backlight_probe = mbp_sysfs_backlight_probe,
+    .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
+    /* .evdev_identify = evdev_is_wellspring3, */
+  },
+
   /* MacBook machines */
 
   {  /* MacBook1,1 (Core Duo) */
@@ -309,6 +319,14 @@ struct machine_ops mb_mops[] = {
     /* .evdev_identify = evdev_is_geyser4hf, */
   },
 
+  {  /* MacBook5,1 (Core2 Duo, October 2008) */
+    .type = MACHINE_MACBOOK_5,
+    .lcd_backlight_probe = mbp_sysfs_backlight_probe,
+    .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
+    /* .evdev_identify = evdev_is_wellspring3, */
+  },
+
   /* MacBook Air machines */
 
   {  /* MacBookAir1,1 (January 2008) */
@@ -317,6 +335,14 @@ struct machine_ops mb_mops[] = {
     .lcd_backlight_step = gma950_backlight_step,
     .lcd_backlight_toggle = gma950_backlight_toggle,
     /* .evdev_identify = evdev_is_wellspring, */
+  },
+
+  {  /* MacBookAir2,1 (October 2008) */
+    .type = MACHINE_MACBOOKAIR_2,
+    .lcd_backlight_probe = mbp_sysfs_backlight_probe,
+    .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
+    /* .evdev_identify = evdev_is_wellspring3, */
   }
 };
 #endif /* __powerpc__ */
@@ -383,24 +409,31 @@ logdebug(char *fmt, ...)
 void
 kbd_set_fnmode(void)
 {
-  int errno1;
+  char *fnmode_node[] =
+    {
+      "/sys/module/hid_apple/parameters/fnmode", /* 2.6.28 & up */
+      "/sys/module/hid/parameters/pb_fnmode",    /* 2.6.20 & up */
+      "/sys/module/usbhid/parameters/pb_fnmode"
+    };
   FILE *fp;
+  int i;
 
   if ((general_cfg.fnmode < 1) || (general_cfg.fnmode > 2))
     general_cfg.fnmode = 1;
 
-  fp = fopen(KBD_FNMODE_FILE, "a");
-  if (fp == NULL)
+  for (i = 0; i < sizeof(fnmode_node) / sizeof(*fnmode_node); i++)
     {
-      errno1 = errno;
+      logdebug("Trying %s\n", fnmode_node[i]);
 
-      fp = fopen(KBD_FNMODE_FILE2620, "a");
-      if (fp == NULL)
-        {
-          logmsg(LOG_INFO, "Could not open %s: %s", KBD_FNMODE_FILE, strerror(errno1));
-          logmsg(LOG_INFO, "Could not open %s: %s", KBD_FNMODE_FILE2620, strerror(errno));
-          return;
-        }
+      fp = fopen(fnmode_node[i], "a");
+      if (fp != NULL)
+	break;
+
+      if (errno == ENOENT)
+	continue;
+
+      logmsg(LOG_INFO, "Could not open %s: %s", fnmode_node[i], strerror(errno));
+      return;
     }
 
   fprintf(fp, "%d", general_cfg.fnmode);
@@ -639,6 +672,9 @@ check_machine_dmi(void)
   /* Core2 Duo MacBook Pro 15" & 17" (February 2008) */
   else if (strcmp(buf, "MacBookPro4,1") == 0)
     ret = MACHINE_MACBOOKPRO_4;
+  /* Core2 Duo MacBook Pro 15" & 17" (October 2008) */
+  else if (strcmp(buf, "MacBookPro5,1") == 0)
+    ret = MACHINE_MACBOOKPRO_5;
   /* Core Duo MacBook (May 2006) */
   else if (strcmp(buf, "MacBook1,1") == 0)
     ret = MACHINE_MACBOOK_1;
@@ -651,9 +687,15 @@ check_machine_dmi(void)
   /* Core2 Duo MacBook (February 2008) */
   else if (strcmp(buf, "MacBook4,1") == 0)
     ret = MACHINE_MACBOOK_4;
+  /* Core2 Duo MacBook (October 2008) */
+  else if (strcmp(buf, "MacBook5,1") == 0)
+    ret = MACHINE_MACBOOK_5;
   /* MacBook Air (January 2008) */
   else if (strcmp(buf, "MacBookAir1,1") == 0)
     ret = MACHINE_MACBOOKAIR_1;
+  /* MacBook Air (October 2008) */
+  else if (strcmp(buf, "MacBookAir2,1") == 0)
+    ret = MACHINE_MACBOOKAIR_2;
   else
     logmsg(LOG_ERR, "Unknown Apple machine: %s", buf);
 
@@ -668,8 +710,8 @@ check_machine_dmi(void)
 static void
 usage(void)
 {
-  printf("pommed v" M_VERSION " ($Rev: 494 $) Apple laptops hotkeys handler\n");
-  printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>\n");
+  printf("pommed v" M_VERSION " ($Rev: 540 $) Apple laptops hotkeys handler\n");
+  printf("Copyright (C) 2006-2009 Julien BLACHE <jb@jblache.org>\n");
 
   printf("Usage:\n");
   printf("\tpommed\t-- start pommed as a daemon\n");
@@ -692,6 +734,7 @@ main (int argc, char **argv)
   int c;
 
   FILE *pidfile;
+  struct utsname sysinfo;
 
   machine_type machine;
 
@@ -709,8 +752,8 @@ main (int argc, char **argv)
 	    break;
 
 	  case 'v':
-	    printf("pommed v" M_VERSION " ($Rev: 494 $) Apple laptops hotkeys handler\n");
-	    printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>\n");
+	    printf("pommed v" M_VERSION " ($Rev: 540 $) Apple laptops hotkeys handler\n");
+	    printf("Copyright (C) 2006-2009 Julien BLACHE <jb@jblache.org>\n");
 
 	    exit(0);
 	    break;
@@ -725,7 +768,7 @@ main (int argc, char **argv)
 
   if (geteuid() != 0)
     {
-      logmsg(LOG_ERR, "pommed needs root privileges to operate\n");
+      logmsg(LOG_ERR, "pommed needs root privileges to operate");
 
       exit(1);
     }
@@ -735,8 +778,8 @@ main (int argc, char **argv)
       openlog("pommed", LOG_PID, LOG_DAEMON);
     }
 
-  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 494 $) Apple laptops hotkeys handler");
-  logmsg(LOG_INFO, "Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>");
+  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 540 $) Apple laptops hotkeys handler");
+  logmsg(LOG_INFO, "Copyright (C) 2006-2009 Julien BLACHE <jb@jblache.org>");
 
   /* Load our configuration */
   ret = config_load();
@@ -785,6 +828,16 @@ main (int argc, char **argv)
       exit(1);
     }
 
+  if (debug)
+    {
+      ret = uname(&sysinfo);
+
+      if (ret < 0)
+	logmsg(LOG_ERR, "uname() failed: %s", strerror(errno));
+      else
+	logdebug("System: %s %s %s", sysinfo.sysname, sysinfo.release, sysinfo.machine);
+    }
+
   ret = evloop_init();
   if (ret < 0)
     {
