@@ -1,5 +1,5 @@
 /*
- * $Id: power.h 482 2008-06-03 17:19:03Z jblache $
+ * $Id: power.h 511 2008-10-18 15:51:22Z jblache $
  */
 
 #ifndef __POWER_H__
@@ -14,6 +14,12 @@
 
 #define POWER_TIMEOUT 200
 
+#ifdef __powerpc__
+# define SYSFS_POWER_AC_STATE  "/sys/class/power_supply/pmu-ac/online"
+#else
+# define SYSFS_POWER_AC_STATE  "/sys/class/power_supply/ADP1/online"
+#endif
+
 
 void
 power_init(void);
